# -*- coding: utf-8 -*-

from django.contrib import admin
from django.utils.translation import ugettext as _
from django.utils.translation import ugettext_lazy as ugl
from django import forms
from django.conf import settings

from jumbo.admin.options import ExtendibleModelAdmin, OrderedModelAdmin, SettingsAdmin
from jumbo.forms.models import AdminModelForm
from jumbo.core.urlresolvers import reverse
from jumbo.middleware.thread_local import get_request
from jumbo.apps.autocomplete import autocomplete
from jumbo.apps.autocomplete import AutocompleteModelChoiceField

from organization.models import *
from organization.views import get_autocomplete_contact_organization

### FORM ###
class OrganizationAdminForm(AdminModelForm):
    class Meta:
        model = Organization

    def __init__(self, *args, **kws):
        super(OrganizationAdminForm, self).__init__(*args, **kws)
        category = get_request().GET.get('category', None)
        instance = kws.get('instance', None)
        ## form ADD
        if category and not instance:
            self.fields['organization_type'].queryset = OrganizationType.objects.filter(category=category)
        ## form EDIT
        elif instance and instance.organization_type:
            category = instance.organization_type.category
            self.fields['organization_type'].queryset = OrganizationType.objects.filter(category=category)


    def clean_vat_number(self):
        """
        Verify input vat number
        """
        data = self.cleaned_data['vat_number']
        if data and len(data) != 11:
            raise forms.ValidationError(_("Vat Number must have length of 11."))
        return data

    def clean_ssn(self):
        """
        Verify input SSN
        """
        from django.contrib.localflavor.it.util import ssn_check_digit
        data = self.cleaned_data['ssn']
        if data and len(data) == 16:
            check = ssn_check_digit(data[0:15])
            if check != data[15]:
                raise forms.ValidationError(_("Invalid SSN code"))
        return data

class ContactAdminForm(AdminModelForm):
    class Meta:
        model = Contact

    class Media:
        js = ('organization/js/contact_form.js',)

    def __init__(self, *args, **kws):
        super(ContactAdminForm, self).__init__(*args, **kws)
        category = get_request().GET.get('category', None)
        instance = kws.get('instance', None)
        self.fields['organization'] = AutocompleteModelChoiceField(
            'contact_organization', label=_("Organization"),
            on_select_token='on_select_organization',
            on_delete_token='on_delete_organization',
        )
#        ## form ADD
#        if category and not instance:
#            self.fields['organization'].queryset = Organization.objects.filter(organization_type__category=category)
#        ## form EDIT
#        elif instance:
#            try:
#                category = instance.organization.organization_type.category
#                self.fields['organization'].queryset = Organization.objects.filter(organization_type__category=category)
#            except:
#                self.fields['organization'].queryset = Organization.objects.all()

    def clean(self):
        cleaned_data = self.cleaned_data
        phone = cleaned_data.get('phone' or None)
        mobile = cleaned_data.get('mobile' or None)
        print phone, mobile
        if not phone and not mobile:
            print "qui"
            raise forms.ValidationError(_("Inserire almeno un numero di telefono valido"))
        return cleaned_data

### INLINE ###
class ContactInline(admin.StackedInline):
    model = Contact
    extra = 0
    exclude = ['date_create', 'date_last_modify','creator','last_modifier',]
    can_delete = True
    fieldsets = (
        (_('general information'), {
            'fields': (
                'status',
                'role',
                ('last_name', 'first_name'),
                ('phone', 'fax',),
                ('mobile', 'email'),
            )
        }),
    )

### SEARCH FORM ###
STATUS = (('', '---'), ('1', ugl('Active')), ('0', ugl('Disactive')),)
class OrganizationAdminSearchForm(forms.ModelForm):
    class Meta:
        model = Organization
        fields = ('organization_type',)
        display_fields = (
            ('status', 'organization_type',),
            'name__icontains',
            ('contacts__last_name__icontains', 'contacts__first_name__icontains'),
            'organization_type'
        )

    def __init__(self, *args, **kws):
        super(OrganizationAdminSearchForm, self).__init__(*args, **kws)
        ORGTYPE_DEF = [('', '---')] + list(ORGANIZATIONTYPE_DEF)
        self.fields['name__icontains']  = forms.CharField(label=_('Name'), required=False)
        self.fields['status'] = forms.ChoiceField(label=_("Status"), choices=STATUS, required=False)
        self.fields['contacts__last_name__icontains'] = forms.CharField(label=_('Contact Last name'), required=False)
        self.fields['contacts__first_name__icontains'] = forms.CharField(label=_('Contact First name'), required=False)

class ContactAdminSearchForm(forms.ModelForm):
    class Meta:
        model = Contact
        fields = ('role',)
        display_fields = (
            ('status', 'organization__organization_type__category'),
            ('role', 'last_name__icontains', 'first_name__icontains'),
            ('organization__name__icontains',),
            'email__icontains',
        )

    def __init__(self, *args, **kws):
        super(ContactAdminSearchForm, self).__init__(*args, **kws)
        ORGTYPE_DEF = [('', '---')] + list(ORGANIZATIONTYPE_DEF)
        self.fields['organization__organization_type__category'] = forms.ChoiceField(label=_('Category'), choices=ORGTYPE_DEF, required=False)
        self.fields['last_name__icontains'] = forms.CharField(label=_('Last Name'), required=False)
        self.fields['first_name__icontains'] = forms.CharField(label=_('First Name'), required=False)
        self.fields['organization__name__icontains'] = forms.CharField(label=_('Organization'), required=False)
        self.fields['status'] = forms.ChoiceField(label=_("Status"), choices=STATUS, required=False)
        self.fields['email__icontains'] = forms.CharField(label=_("Email"), required=False)

### GENERIC ###
class OrganizationAdmin(ExtendibleModelAdmin):
    class Media:
        css = {
            "all" : ("organization/css/admin_organization.css",)
        }
        js = ("%sasterisk/js/click_to_call.js" % settings.MEDIA_URL,)
    form = OrganizationAdminForm
    save_on_top = True
    fieldsets = (
        (_('logs admin'), {
            'fields':('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general information'), {'fields':(
            'status',
            'name',
            'organization_type',
            ('phone', 'fax'),
        )}),
    )
    inlines = [ContactInline,]
    list_display = (
        'get_status_icon', 'name',
        'get_phone', 'fax', 'get_organization_type',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )
    list_display_csv = (
        'get_status_display', 'name', 'phone', 'fax', 'organization_type',
    )
    search_form = OrganizationAdminSearchForm
    search_fields = ('name',)

    def get_link_url(self, obj):
        """
        Returns link to organization URL
        """
        if obj.url:
            return """<a href=%s target="_blank">%s</a>""" % (obj.url, obj.url)
        return ""
    get_link_url.short_description = _("Url")
    get_link_url.allow_tags = True

    def get_icon_link_url(self, obj):
        """
        Returns link icon to organization URL
        """
        if obj.url:
            return """
                <a href=%(href)s target="_blank">
                    <img src='%(url)sadmin/images/web.png' alt='%(title)s' title='%(title)s'/>
                </a>""" % {
                    'href': obj.url,
                    'url': settings.MEDIA_URL,
                    'title': obj.url
                }
        return ""
    get_icon_link_url.short_description = _("www")
    get_icon_link_url.allow_tags = True

    def get_link_email(self, obj):
        """
        Returns link to organization email
        """
        if obj.email:
            return """<a href=mailto:%s>%s</a>""" % (obj.email, obj.email)
        return ""
    get_link_email.short_description = _("Email")
    get_link_email.allow_tags = True

    def get_icon_link_email(self, obj):
        """
        Returns link icon to organization email
        """
        if obj.email:
            return """
                <a href=mailto:%(email)s>
                    <img src='%(url)sadmin/images/email.png' alt='%(title)s' title='%(title)s'/>
                </a>""" % {
                    'email': obj.email,
                    'url': settings.MEDIA_URL,
                    'title': obj.email
                }
        return ""
    get_icon_link_email.short_description = _("Mail")
    get_icon_link_email.allow_tags = True

    def get_link_organization_detail(self, obj):
        """
        Return link to organization detail
        """
        return """<a href='%(reverse_url)s' title='%(title)s'>%(name)s</a>""" % {
            'reverse_url': reverse('admin:organization_organization_detail', obj.id),
            'title': _("View Detail"),
            'name': obj.name
        }
    get_link_organization_detail.short_description = _("Organization")
    get_link_organization_detail.order_admin_field = 'name'
    get_link_organization_detail.allow_tags = True

    def get_organization_type(self, result):
        """
        Returns organization type name
        """
        return "<b>%s</b>" % (result.organization_type.name,)
    get_organization_type.short_description = _("Type")
    get_organization_type.order_admin_field = 'organization_type__category__name'
    get_organization_type.allow_tags = True

    if 'asterisk' in settings.INSTALLED_APPS:
        def get_phone(self, result):
            """
            Returns link to click to call phone
            """
            from asterisk.utils import click_to_call
            return click_to_call(result.phone, get_link=True)
        get_phone.short_description = _("P1")
    else:
        def get_phone(self, result):
            """
            Returns phone number
            """
            return  result.phone
        get_phone.short_description = _("Phone")
    get_phone.order_admin_field = 'phone'
    get_phone.allow_tags = True

    if 'asterisk' in settings.INSTALLED_APPS:
        def get_phone2(self, result):
            """
            Returns link to click to call phone 2
            """
            from asterisk.utils import click_to_call
            return click_to_call(result.phone2, get_link=True)
        get_phone2.short_description = _("P2")
    else:
        def get_phone2(self, result):
            """
            Returns phone 2 number
            """
            return  result.phone2
        get_phone2.short_description = _("Phone 2")
    get_phone2.order_admin_field = 'phone2'
    get_phone2.allow_tags = True

class OrganizationTypeAdmin(ExtendibleModelAdmin):
    class Media:
        css = {
            "all" : ("organization/css/admin_organization.css",)
        }

    save_on_top = True
    fieldsets = (
        (_('logs admin'), {
            'fields':('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general information'), {'fields': ('status', 'category', 'name')}),
    )
    list_display = (
        'get_status_icon', 'name', 'category',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )


class ContactRoleAdmin(ExtendibleModelAdmin):
    class Media:
        css = {
            "all" : ("organization/css/admin_organization.css",)
        }

    save_on_top = True
    fieldsets = (
        (_('logs admin'), {
            'fields':('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general information'), {'fields': ('status', 'name', 'ordering')}),
    )
    list_display = (
        'get_status_icon', 'name',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )

class ContactAdmin(ExtendibleModelAdmin):
    class Media:
        css = {
            "all" : ("organization/css/admin_organization.css",)
        }
        js = ("%sasterisk/js/click_to_call.js" % settings.MEDIA_URL,)

    save_on_top = True
    fieldsets = (
        (_('logs admin'), {
            'fields':('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general information'), {
            'fields': (
                'status',
                ('organization', 'role',),
                ('last_name', 'first_name'),
                ('phone', 'fax',),
                ('mobile', 'email'),
            )
        }),
    )
    list_display = (
        'get_status_icon', 'last_name', 'first_name', 'role',
        'get_organization_phone', 'get_phone', 'get_mobile', 'fax', 'get_icon_link_email',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )
    list_display_csv = (
        'get_status_display', 'last_name', 'first_name', 'role', 'organization', 'phone', 'mobile',
        'fax'
    )
    form = ContactAdminForm
    search_form = ContactAdminSearchForm
    search_fields = ('last_name', 'first_name')
    default_group_by = 'contact_group_by'
    ordering = ('last_name', 'first_name')

    def contact_group_by(self, obj):
        return """
            %s - %s
        """ % (
            obj.organization.name,
            obj.organization.phone
        )
    contact_group_by.short_description = _('Organization')
    contact_group_by.allow_tags = True
    contact_group_by.admin_order_field = ('organization__name')

    if 'asterisk' in settings.INSTALLED_APPS:
        def get_organization_phone(self, obj):
            from asterisk.utils import click_to_call
            if obj.organization:
                return click_to_call(obj.organization.phone, get_link=True)
            return ""
        get_organization_phone.short_description = _("Organization Phone")
        get_organization_phone.allow_tags = True
    else:
        def get_organization_phone(self, obj):
            return obj.organization.phone or ""
        get_organization_phone.short_description = _("Organization Phone")
        get_organization_phone.allow_tags = True

    def get_link_email(self, obj):
        """
        Returns link to contact email
        """
        if obj.email:
            return """<a href=mailto:%s>%s</a>""" % (obj.email, obj.email)
        return ""
    get_link_email.short_description = _("Email")
    get_link_email.allow_tags = True

    def get_icon_link_email(self, obj):
        """
        Returns icon link to contact email
        """
        if obj.email:
            return """
                <a href=mailto:%(email)s>
                    <img src='%(url)sadmin/images/email.png' alt='%(title)s' title='%(title)s'/>
                </a>
                """ % {
                    'email': obj.email,
                    'url': settings.MEDIA_URL,
                    'title': obj.email
                }
        return ""
    get_icon_link_email.short_description = _("Mail")
    get_icon_link_email.allow_tags = True

    if 'asterisk' in settings.INSTALLED_APPS:
        def get_phone(self, obj):
            from asterisk.utils import click_to_call
            return click_to_call(obj.phone, get_link=True)
        get_phone.short_description = _("P")
    else:
        def get_phone(self, obj):
            return  obj.phone or ''
        get_phone.short_description = _("Phone")
    get_phone.order_admin_field = 'phone'
    get_phone.allow_tags = True

    if 'asterisk' in settings.INSTALLED_APPS:
        def get_mobile(self, obj):
            from asterisk.utils import click_to_call
            return click_to_call(obj.mobile, get_link=True)
        get_mobile.short_description = _("M")
    else:
        def get_mobile(self, obj):
            return  obj.mobile or ''
        get_mobile.short_description = _("Mobile")
    get_mobile.order_admin_field = 'mobile'
    get_mobile.allow_tags = True

admin.site.register(Organization, OrganizationAdmin)
admin.site.register(OrganizationType, OrganizationTypeAdmin)
admin.site.register(ContactRole, ContactRoleAdmin)
admin.site.register(Contact, ContactAdmin)
autocomplete.register(
    'contact_organization', model = Organization,
    views = get_autocomplete_contact_organization
)