# -*- coding: utf-8 -*-
from datetime import date, timedelta

from django.conf import settings
from django.db import models, connection
from django.db.models import Avg, Sum, Min, Max
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext
from django.contrib.auth.models import User
from django.core.exceptions import ValidationError

from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel, DefaultModel
from jumbo.db.models import Settings as JSettings
from jumbo.apps.abicab.models import Bank as DetailBank
from jumbo.apps.i18n.models import Language
from jumbo.utils.functions import get_settings


class Organization (DateModel, UserModel, StatusModel, OrderedModel):
    """
    Stores organization
    """
    organization_type = models.ForeignKey (
        "OrganizationType",
        db_index = True,
        blank = True, null = True,
        related_name = "organizations",
        verbose_name = _("Organization type"),
    )
    name = models.CharField (
        max_length = 100,
        db_index = True,
        verbose_name = _("name"),
    )
    url = models.URLField (
        max_length=100,
        blank = True, null = True,
        verify_exists = True,
        verbose_name = _("url"),
    )
    email = models.EmailField (
        max_length = 100,
        blank = True, null = True,
        verbose_name = _("email"),
    )
    phone = models.CharField (
        max_length = 20,
        blank = True, null = True,
        verbose_name = _("phone"),
    )
    phone2 = models.CharField (
        blank = True, null = True,
        max_length = 20,
        verbose_name = _("phone2"),
    )
    fax = models.CharField (
        blank = True, null = True,
        max_length = 20,
        verbose_name = _("fax"),
    )
    note = models.TextField (
        blank = True, null = True,
        verbose_name = _("note"),
    )
    ldap = models.BooleanField(
        default=False,
        verbose_name = _("LDAP"),
    )

    class Meta:
        verbose_name = _("organization")
        verbose_name_plural = _("organizations")
        permissions = (
            ("view_organization",   "Can view organizations"),
            ("save_ldap",           "Can save in phone addreebook"),
        )
        ordering = ['name']

    def __unicode__(self):
        return u"%s" % self.name

    def save(self):
        ## modifico il valore del name
        osn = get_settings("organization.settings", "ORGANIZATION_NAME_STYLE")
        if osn == "title":
            self.name = self.name.title()
        elif osn == "lower":
            self.name = self.name.lower()
        elif osn == "upper":
            self.name = self.name.upper()

        super(Organization, self).save()

    def get_contacts(self, other_filters={}):
        """
        Returns all active contacts
        """
        c = self.contacts.filter(status=1)
        if other_filters:
            c = c.filter(**other_filters)
        return c

    def get_all_mails(self, contact_filter={}):
        """
        Returns organization email, contacts emails and agents email.
        Example:
        (
            ('info@thundersystems.it (Organization)', 'info@thundersystems.it'),
            ('vparenti@thundersystems.it (Contact)', 'vparenti@thundersystems.it'),
            …
        )
        """
        mails = []

        # Organization email
        if self.email:
            label_mail = ugettext("%(email)s (Organization)" % {'email':self.email})
            choice_mail = (self.email, label_mail)
            if choice_mail not in mails:
                mails.append(choice_mail)

        # All contacts emails
        for c in self.get_contacts(other_filters=contact_filter):
            if c.email:
                label_mail = ugettext("%(email)s (Contact)" % {'email':c.email})
                choice_mail = (c.email, label_mail)
                if choice_mail not in mails:
                    mails.append(choice_mail)

        # All agents emails
        for a in self.get_agents():
            if a.email:
                label_mail = ugettext("%(email)s (Agent)" % {'email':a.email})
                choice_mail = (a.email, label_mail)
                if choice_mail not in mails:
                    mails.append(choice_mail)

        return mails

####################################
# TIPO ANAGRAFICA                  #
####################################
ORGANIZATIONTYPE_DEF = (
    ("com", _("company"),),
    ("cus", _("customer"),),
    ("sup", _("supplier"),),
)

class OrganizationType (DateModel, UserModel, StatusModel, OrderedModel):
    """
    Stores organization's types
    EXAMPLE: 'end user', 'retailer', 'dealer'
    """
    category = models.CharField (
        max_length = 15,
        choices = ORGANIZATIONTYPE_DEF,
        db_index = True,
        verbose_name = _("category"),
    )
    name = models.CharField (
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _("name"),
    )

    class Meta:
        verbose_name = _("organization type")
        verbose_name_plural = _("organization types")
        ordering = ('ordering', 'name')

    def __unicode__(self):
        return u"%s" % self.name

###################################
# CONTATTO RUOLO                  #
###################################
class ContactRole (DateModel, UserModel, StatusModel, OrderedModel):
    """
    Stores contact roles
    EXAMPLE: 'sales manager', 'development manager'
    """
    name = models.CharField (
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _("name"),
    )

    class Meta:
        verbose_name = _("Contact role")
        verbose_name_plural = _("Contact roles")
        ordering = ('name',)

    def __unicode__(self):
        return u"%s" % self.name

##############################
# CONTATTO                   #
##############################
class Contact (DateModel, UserModel, StatusModel, OrderedModel):
    """
    Stores contacts
    """
    ## ForeignKey
    organization = models.ForeignKey (
        Organization,
        db_index = True,
        related_name='contacts',
        verbose_name = _("Organization"),
    )
    role = models.ForeignKey (
        ContactRole,
        blank = True, null = True,
        related_name='contacts',
        verbose_name = _("Role"),
    )
    last_name = models.CharField (
        max_length = 100,
        verbose_name = _("Last Name"),
    )
    first_name = models.CharField (
        max_length = 100,
        verbose_name = _("First Name"),
    )
    phone = models.CharField (
        max_length = 20,
        verbose_name = _("Contact Phone"),
        blank = True, null = True
    )
    mobile = models.CharField (
        max_length = 20,
        blank = True, null = True,
        verbose_name = _("Mobile"),
    )
    fax = models.CharField (
        max_length = 20,
        blank = True, null = True,
        verbose_name = _("Fax"),
    )
    email = models.EmailField (
        max_length = 100,
        blank = True, null = True,
        verbose_name = _("Email"),
    )
    note = models.TextField (
        max_length = 255,
        blank = True, null = True,
        verbose_name = _("Note"),
    )
    ldap = models.BooleanField(
        default=False,
        verbose_name = _("LDAP"),
    )

    def get_full_name(self):
        """
        returns the contact's full name (last name and first name)
        """
        return "%s %s" % (self.last_name, self.first_name)
    full_name = property(get_full_name)

    class Meta:
        verbose_name = _("contact")
        verbose_name_plural = _("contacts")
        permissions = (
            ("view_contact"   , "Can view contacts"),
        )
        unique_together = ('phone', 'mobile',)
        ordering = ['ordering', 'last_name', 'first_name']

    def save(self):
        ## modifico il valore del last_name e del first_name
        osn = get_settings("organization.settings", "CONTACT_NAME_STYLE")
        if osn == "title":
            self.last_name = self.last_name.title()
            self.first_name = self.first_name.title()
        elif osn == "lower":
            self.last_name = self.last_name.lower()
            self.first_name = self.first_name.lower()
        elif osn == "upper":
            self.last_name = self.last_name.upper()
            self.first_name = self.first_name.upper()

        super(Contact, self).save()

    def __unicode__(self):
        return u"%s %s (%s)" % (self.last_name, self.first_name, self.role)

class Settings(JSettings):
    pass